#!/usr/bin/env python
# $Id$
''' Command Line Interface for sabnzbd+ '''
import os, sys
import httplib
from xmltopy import XmlDictConfig 
import xml.etree.ElementTree as ElementTree
import getopt, ConfigParser

CONFIGFILE = os.environ['HOME'] + "/.nzbrc"

class SabnzbdAutomation(object):
    ''' Sabnzbd Automation Class '''
    def __init__(self, config = None):
        if not config:
            config = { 'host' : 'localhost', 'port' : '8080', 
                       'username' : None, 'password;' : None,
                       'apikey' : None }, 
        # Private Variables
        self.__fetchpath = '/sabnzbd/api?apikey=' + config ['apikey'] + '&priority=1&mode='
        self.__servername = config['host'] + ':' + config['port']
        self.__header = { 'User-Agent' : 'SabnzbdAutomation' }
        # Setup authentication if needed.
        if config['username'] and config['password']:
            self.__postdata = 'ma_password=' + config['password'] + '&' +\
                              'ma_username=' + config['username']
            self.__header['Content-type'] = 'application/x-www-form-urlencoded'
        else:
            self.__postdata = None
        
        # Public Variables
        self.command_status = 'ok'
        self.queue_status = {}
        self.url = 'http://' + self.__servername + '/sabnzbd/'

    # Private Methods
    def __evalreturn(self, retval, command = 'Null'):
        ''' Convert web output to boolean value '''
        # Web output should only be 'ok\n' or 'error\n'
        retval = retval.lower().strip('\n')
        self.command_status = command + ' -> ' + retval
        if retval == 'ok' :
            return True
        if retval == 'error':
            return False

    def __parse_status(self, status_obj):
        ''' Convert status object to python dictionary '''
        try:
            status_obj = status_obj.replace('&', '&amp;') 
            root = ElementTree.XML(status_obj.strip())
            self.queue_status = XmlDictConfig(root)
            self.command_status = 'qstatus -> ok'
        except ValueError:
            self.command_status = 'qstatus -> error'
            return False
        return True

    def __send_request(self, command, path):
        ''' Send command to server '''
        try:
            conn = httplib.HTTPConnection(self.__servername)
            if self.__postdata:
                conn.request('POST', path, self.__postdata, self.__header)
            else:
                conn.request('GET', path, None, self.__header)
            try:
                response = conn.getresponse()
            except httplib.BadStatusLine:
                if command == 'shutdown':
                    self.command_status = command + ' -> ok'
                    return True
                else:
                    raise Exception
            if response.status != 200:
                msg = str(response.status) + ':' + response.reason
                raise httplib.HTTPException(msg)
            conn.close()
        except httplib.HTTPException, err:
            self.command_status = 'http://' + self.__servername + path + \
                                  ' -> ' + err.message
            return False
        except:
            self.command_status = "Cannot connect to " + \
                                  self.__servername + path
            return False
        
        if command != 'qstatus':
            return self.__evalreturn(response.read(), command)
        else:
            return self.__parse_status(response.read())

    # Public Methods
    def send_command(self, command, arg = None, job_option = '3'):
        ''' http://sabnzbdplus.wiki.sourceforge.net/Automation+Support '''
        
        url_fragment = command
        if command in ('addfile', 'addurl', 'addid'):
            url_fragment += '&name=' + arg + '&pp=' + job_option
        elif command == 'qstatus':
            url_fragment += '&output=xml'
        elif command == 'autoshutdown':
            if arg not in ('0', '1'):
                return False
            else:
                url_fragment += '&name=' + arg

        self.url = 'http://' + self.__servername + \
                    self.__fetchpath + url_fragment
        return self.__send_request(command, self.__fetchpath + url_fragment) 
       
    def print_status(self):
        ''' Print pretty table with status info '''
        width = 80
        print self.url
        print 'Temp/Target Disk: %.2f / %.2f GB\t\t\tSpeed: ' % \
                ( float(self.queue_status['diskspace1']),
                  float(self.queue_status['diskspace2']) ),
        if self.queue_status['paused'] == 'True':
            print "Paused"
        else:
            print "%.2f kb/s" %  float(self.queue_status['kbpersec'])
        print 'Queue: %.2f / %.2f MB'.center(width) % \
                ( float(self.queue_status['mb']) - 
                    float(self.queue_status['mbleft']),
                  float(self.queue_status['mb']) )
        print "Filename\t\t\t\t\t\t\t       Size (MB)"
        print '-' * width
        if self.queue_status['jobs'] != '\n':
            try:
                self.queue_status['jobs']['job'].keys()
                jobs = [ self.queue_status['jobs']['job'] ]
            except AttributeError:
                jobs = self.queue_status['jobs']['job']
            for each in jobs:
                print "%s\t\t\t%.2f / %.2f" % \
                    ( each['filename'].ljust(40), 
                      float(each['mbleft']), 
                      float(each['mb']) )
        print
        return True

def usage(exitval):
    ''' Usage '''
    msg = sys.argv[0].split('/').pop()
    msg += " <command> <args>\n\n"
    msg += "Commands:\n\tpause\n\tresume\n\tshutdown\n\tstatus\n"
    msg += "\tnewzbin <id>\n\turlget <nzb url>\n"
    sys.stderr.write(msg + '\n')
    sys.exit(exitval)

def parse_options(options):
    ''' Parse Cli options '''
    default_opts = ("hj:" , ["help" , "job-option="])
    command = None
    command_arg = None
    job_option = '3'
    try:
        opt , args = getopt.getopt(options, default_opts[0], default_opts[1])
    except getopt.GetoptError:
        usage(2)
    for option , arguement in opt:
        if option in ("-h", "--help"):
            usage(2)
        elif option in ("-j", "--job-option="):
            job_option = str(arguement)

    # Commands
    if 'pause' in args:
        command = 'pause'
    elif 'resume' in args:
        command = 'resume'
    elif 'shutdown' in args:
        command = 'shutdown'
    elif 'status' in args:
        command = 'qstatus'
    elif 'newzbin' in args:
        command = 'addid'
        command_arg = str(args[args.index('newzbin')+1])
    elif 'urlget' in args:
        command = 'addurl'
        command_arg = str(args[args.index('urlget')+1])
    else:
        usage(2)

    return (command, command_arg, job_option)

def parse_config(config):
    ''' Parse config file for server info '''
    parser = ConfigParser.ConfigParser()
    config_dict = {}
    try:
        config_file = open(config)
        parser.readfp(config_file)
        config_file.close()
    except IOError:
        sys.stderr.write('Unable to open ' + config + '\n')
        usage(1)

    try:
        for each in ('host', 'port', 'username', 'password', 'apikey'):
            config_dict[each] = parser.get('server', each)
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
        sys.stderr.write('Unable to parse ' + config + '\n')
        sys.stderr.write('Format should be:\n')
        sys.stderr.write('[server]\n')
        sys.stderr.write('host = <sabnzbd server>\n')
        sys.stderr.write('port = <sabnzbd port>\n')
        sys.stderr.write('username = <sabnzbd name> | None\n')
        sys.stderr.write('password = <sabnzbd password> | None\n') 
        sys.stderr.write('apikey = <sabnzbd apikey> | None\n') 
        usage(1)

    return config_dict

def main():
    ''' Command line front end to sabnzbd+ '''
    config_dict = parse_config(CONFIGFILE)
    command, command_arg, job_option = parse_options(sys.argv[1:])
    sabnzbd = SabnzbdAutomation(config_dict)
    if sabnzbd.send_command(command, command_arg, job_option):
        if command == 'qstatus':
            sabnzbd.print_status()
        else:
            print sabnzbd.url
    print sabnzbd.command_status

if __name__ == '__main__':
    main()
    sys.exit(0)

# vim:syntax=python
# vim:sw=4:softtabstop=4:expandtab
